	INCLUDE EFR32MG12.inc		; CPU register definitions
	INCLUDE terminal.inc		; terminal function definitions	
	INCLUDE	spi.inc	
	INCLUDE	icm20648.inc		
	EXPORT	main
	IMPORT	GPIO_setup		; import names from other code files
	IMPORT	LETIMER_setup	
	IMPORT	SPI3_setup	

ORDER	EQU	4			; average 2^ORDER readings
	
;-------------------------------------	; non-initialized DATA segment in RAM 
	AREA	RAM, DATA, NOINIT, READWRITE, ALIGN=3
Xacc	SPACE	2			; accelerometer data
Yacc	SPACE	2
Zacc	SPACE	2	
	EXPORT	Xacc

F_ACC 	SPACE 	3*2*(2<<ORDER)		; moving ACC average filter buffer (16-bit values)
FA_Sums	SPACE	3*4			; filter average sum buffer
	
;-------------------------------------	; CODE segment in flash (ROM) 
	AREA    |.text|, CODE, READONLY
main	PROC				; main user code
	bl	Terminal_setup		; setup TeraTerm communication
	bl	GPIO_setup		; configure port pins
	bl	LETIMER_setup
	bl	SPI3_setup
	
	bl	ICM20648_setup	
	bl	Init_Filter
	
loop					
	wfi
	bl	ICM20648_read_ACC
	bl	Print_raw_acc_data	

;	bl	Print_filter_data
	bl	NewLine		
	b	loop			; repeat periodic task		
	ENDP

;-----------------------------------------------------------------
Init_Filter	PROC	
	ldr	R8, =F_ACC		; pointer to filter data
	movs	R0, #0
	movs	R1, #(3*(2<<ORDER))
zl1	strh	R0, [R8], #2		; zero ACC filter data
	subs	R1, #1
	bge	zl1
	
	ldr	R9, =FA_Sums
	movs	R1, #3
zl2	str	R0, [R9], #4		; zero ACC average sums	
	subs	R1, #1
	bge	zl2
		
	movs	R10, #0			; filter buffer index
	bx	LR
	ENDP
		
;----------------------------------------------------------------
Print_raw_acc_data	PROC
	push	{LR, R0, R1}
	ldr	R1, =Xacc		; print Xacc
	ldrsh	R0, [R1]
	bl	WriteInt
	mov	R0, #0x09
	bl	WriteChar
	ldrsh	R0, [R1, #2]		; print Yacc
	bl	WriteInt
	mov	R0, #0x09
	bl	WriteChar	
	ldrsh	R0, [R1, #4]		; print Zacc
	bl	WriteInt
	mov	R0, #0x09
	bl	WriteChar
	mov	R0, #0x09
	bl	WriteChar	
	pop	{PC, R0, R1}
	ENDP
		
;----------------------------------------------------------------
Print_filter_data	PROC
	push	{LR, R0, R1, R7-R9}
	ldr	R7, =Xacc
	ldr	R8, =F_ACC
	ldr	R9, =FA_Sums
	bl	Update_ACC_Filter	; update ACC X-filter
	bl	WriteInt		; print ACC X filtered value
	movs	R0, #0x09		; TAB char
	bl	WriteChar
	bl	Update_ACC_Filter	; update ACC Y-filter
	bl	WriteInt		; print ACC Y filtered value
	movs	R0, #0x09		; TAB char
	bl	WriteChar
	bl	Update_ACC_Filter	; update ACC Z-filter
	bl	WriteInt		; print ACC Z filtered value
	
	add	R10, #1			; update filter index
	and	R10, #((1<<ORDER)-1)	; take it modulo filter size
	pop	{PC, R0, R1, R7-R9}
	ENDP
		
;-----------------------------------------------------------------
Update_ACC_Filter	PROC
	ldrsh	R0, [R7], #2		; get sensor data	
	ldrsh	R1, [R8, R10, LSL #1]	; get oldest filter data
	ldr	R2, [R9]		; get filter sum value
	sub	R2, R1			; subtract oldest value from the sum
	add	R2, R0			; add newest value to the sum
	str	R2, [R9], #4		; save filter sum
	strh	R0, [R8, R10, LSL #1]	; save newest data in filter buffer	
	add	R8, #(2<<ORDER)
	mov	R0, R2			; load filter sum to R0
	asr	R0, #ORDER		; divide it by 2^ filter order		
	bx	LR
	ENDP
	
	ALIGN
	END